"""
This module provides admin functionality
"""

import os
import io
import csv
import time
import discord
import zipfile
import matplotlib.pyplot as plt
from discord.ext import commands
from discord.ext.commands import Context


class Admin(commands.Cog, name="admin"):
    """The sync class. Provides the functionality to sync"""

    def __init__(self, bot) -> None:
        self.bot = bot

    @commands.command(
        name="flag",
        description="Sends the desired flag.",
    )
    @commands.has_permissions(administrator=True)
    async def print_flag(self, ctx: Context) -> None:
        """
        Prints the secret flag

        :param context: The command context.
        """

        await ctx.send(os.getenv("FLAG"))

    @commands.command(
        name="stats",
        description="View channel statistics.",
    )
    @commands.has_permissions(administrator=True)
    # Grab all messages in a channel, including id and timestamp.
    async def channel_statistics(self, ctx: Context, channel_name) -> None:
        """
        View channel statistics.

        :param context: The command context.
        :param channel_name: The name of the exfiltration channel
        """
        guild = self.bot.get_guild(self.bot.config["server_id"])
        category = discord.utils.get(guild.categories, name="Exfiltration-channels")
        channel = discord.utils.get(category.channels, name=channel_name)
        if channel is None:
            await ctx.send("Exfiltration channel not found")
            return

        csv_data = io.StringIO()
        csv_writer = csv.writer(csv_data)
        csv_writer.writerow(['id', 'timestamp'])
        timestamps = []
        async for message in channel.history(limit=200):
            timestamps.append(message.created_at)
            csv_writer.writerow([message.id, int(message.created_at.timestamp())])
        csv_contents = csv_data.getvalue()
        csv_data.close()

        hours = [dt.hour for dt in timestamps]

        # Count occurrences of each hour
        hour_counts = {}
        for hour in hours:
            hour_counts[hour] = hour_counts.get(hour, 0) + 1

        # Sort the hours
        sorted_hours = sorted(hour_counts.keys())

        # Create bar chart
        plt.figure(figsize=(10, 6))
        plt.bar(sorted_hours, [hour_counts[hour] for hour in sorted_hours], color='skyblue')

        # Add labels and title
        plt.xlabel('Hour')
        plt.ylabel('# Message')
        plt.title(f'Statistics for {channel_name}')

        # Show plot
        plt.xticks(range(24))
        plt.grid(True)

        img_bytesio = io.BytesIO()
        plt.savefig(img_bytesio, format='png')
        img_bytesio.getvalue()

        zip_data = io.BytesIO()
        with zipfile.ZipFile(zip_data, 'w') as zipf:
            zipf.writestr('data.csv', csv_contents)
            zipf.writestr('chart.png', img_bytesio.getvalue())

        zip_data.seek(0)
        await ctx.send(file=discord.File(zip_data, filename='Statistics.zip'))


async def setup(bot) -> None:  # pylint: disable=missing-function-docstring
    await bot.add_cog(Admin(bot))
